"""Constants for CG submodels."""
#  This file is part of FAST-OAD_CS25
#  Copyright (C) 2022 ONERA & ISAE-SUPAERO
#  FAST is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.

SERVICE_HORIZONTAL_TAIL_CG = "service.cg.horizontal_tail"
SERVICE_VERTICAL_TAIL_CG = "service.cg.vertical_tail"
SERVICE_WING_CG = "service.cg.wing"
SERVICE_FLIGHT_CONTROLS_CG = "service.cg.wing.control_surfaces"
SERVICE_TANKS_CG = "service.cg.tanks"
SERVICE_OTHERS_CG = "service.cg.others"
SERVICE_GLOBAL_CG = "service.cg.global"
SERVICE_MLG_CG = "service.cg.mlg"
SERVICE_AIRCRAFT_CG = "service.cg.aircraft"
SERVICE_EMPTY_AIRCRAFT_CG = "service.cg.empty_aircraft"
SERVICE_LOAD_CASES_CG = "service.cg.load_cases"
